;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Boss Bass
;; by Sonikku
;; Description: A large fish sprite that will lunge at Mario and attempt to eat him.
;; 
;; Although the code isn't perfect and is prone to odd behavior:
;;     If this sprite detects a vertical Layer 2 Scroll generator, it will attempt to move with layer 2.
;;     If this sprite detects a Layer 3 Tide, it will appear and chase at the surface of the water.
;; If layer 1 can vertically scroll, ensure that layer 2 (or 3) has the "constant" scroll setting, as "variable" or "none"
;; can cause odd behavior.
;; 
;; This sprite does not require Sprite Buoyancy to be enabled.
;; 
;; Uses Extra Bit: YES
;; If Extra Bit is set, it will be 16x16 and move slightly slower, and be capable of dying to stomps and unable to eat Mario.

	print "INIT ",pc
SpriteInit:
	LDA $D8,x	; \
	STA $1510,x	;  | setup surface coordinates
	LDA $14D4,x	;  | 
	STA $151C,x	; /
	LDA $1534,x	; \ branch if already cheep cheep
	BNE .small	; /
	LDA $7FAB10,x	; \
	AND #$04	;  | branch if the extra bit isn't set
	BEQ +		; /
.small	INC $1534,x	; mark sprite as the cheep cheep
	LDA $1656,x	; \
	ORA #$10	;  | sprite can be stomped
	STA $1656,x	; /
	LDA $167A,x	; \
	AND #$7B	;  | sprite uses default interaction
	STA $167A,x	; /
+	RTL		; 

	print "MAIN ",pc
SpriteMain:
	PHB
	PHK
	PLB
	JSR BossBass_Main
	PLB
	RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite routine

BossBass_Main:
	JSR SUB_OFF_SCREEN_X0

	LDA $157C,x	; \
	PHA		;  | 
	LDA $B6,x	;  | 
	BEQ +		;  | 
	STZ $157C,x	;  | 
	LDA $B6,x	;  | all this does is make $157C,x be based on x speed, but it only applies graphically
	BMI +		;  | 
	INC $157C,x	;  | 
+	JSR SubGfx	;  | 
	PLA		;  | 
	STA $157C,x	; / 

	LDA $9D		; \ don't run if sprites are locked
	BNE .return	; /

	LDA $C2,x	; \ branch if not being eaten
	BEQ +		; /
	LDA #$C0	; \ put mario's y (high) position at a ridiculously high number
	STA $97		; /
	LDA #$FF	; \ make mario invisible
	STA $78		; /
	STA $18BD	; 
	STA $1404	; 
	STZ $13F1	; 
	STZ $1412	; \ no camera movement
	STZ $1413	; / 
	STZ $7D		; no x speed
	STZ $7B		; no y speed
	LDA $1540,x	; \ branch if timer is non-zero
	BNE +		; /
	JSL $00F606	; kill mario
+
	LDA $14C8,x	; 
	CMP #$08	; 
	BEQ +		; 
.return	RTS
+	JSL $018022	; \ update x/y positions based on speed
	JSL $01801A	; /

	LDA $1BE3	; \ branch if no layer 3 image
	BEQ .l2scroll	; /
	CMP #$03	; \ branch if tile-specific layer 3 image
	BEQ .l2scroll	; /

	REP #$20	; \
	LDA $24		;  | 
	SEC		;  | 
	SBC $1C		;  | 
	EOR #$FFFF	;  | 
	INC		;  | do some math to make the surface coordinate the surface of layer 3 tides
	CLC		;  | 
	ADC #$0100	;  | 
	SEP #$20	;  | 
	STA $1510,x	;  | 
	XBA		;  | 
	STA $151C,x	; /
	BRA .no_l2scroll; 
.l2scroll
	REP #$20	; \
	LDA $144C	;  | branch if layer 2 has no velocity
	SEP #$20	;  | 
	BEQ .no_l2scroll; /
	LDY #$00	; \
	LDA $17BE	;  | 
	EOR #$FF	;  | 
	INC		;  | 
	BPL +		;  | 
	DEY		;  | move sprite's surface coordinate according to layer 2 movement
+	CLC		;  | 
	ADC $1510,x	;  | 
	STA $1510,x	;  | 
	TYA		;  | 
	ADC $151C,x	;  | 
	STA $151C,x	; /
.no_l2scroll
	LDA $1534,x	; \ branch if sprite is Big Bertha
	BEQ .eatmario	; /
	JSL $01A7DC	; default interaction with mario
	BRA .nocontact	; 
.eatmario
	LDA $167A,x	; 
	PHA		; 
	LDA $1490	; \ branch if starman
	BNE .def	; /
	LDA $1594,x	; \
	CMP #$01	;  | branch if lunging
	BEQ +		; /
.def	STZ $167A,x	; default interaction with mario
+	JSL $01A7DC	; \ branch if no contact
	BCC +		; /
	LDA #$01	; \
	STA $C2,x	;  | eat mario
	LDA #$30	;  | 
	STA $1540,x	; /
+	PLA		; 
	STA $167A,x	; 
.nocontact
	INC $1570,x	; increment frame counter
	LDA $D8,x	; \
	STA $0C		;  | sprite's real-y coordinates
	LDA $14D4,x	;  | 
	STA $0D		; /
	LDA $1510,x	; \
	STA $0E		;  | sprite's surface-y coordinates
	LDA $151C,x	;  | 
	STA $0F		; /
	STZ $1602,x	; display non-lunging frame
	LDY $1594,x	; \
	LDA $B6,x	;  | 
	CLC		;  | 
	ADC .yvel_limit,y; | 
	BMI +		;  | accelerate y speed based on state
	LDA .yaccel,y	;  | 
	CLC		;  | 
	ADC $AA,x	;  | 
+	STA $AA,x	; /
	LDA $1594,x	; \ index by eat state
	JSL $0086DF	; /
	dw .swimming
	dw .jumpingup
	dw .resurface
.swimming
	LDA $1510,x	; \
	STA $D8,x	;  | sprite is locked onto surface position
	LDA $151C,x	;  | 
	STA $14D4,x	; /

	JSR SUB_HORZ_POS; \
	LDA $0E		;  | 
	CLC		;  | 
	ADC .x_diff,y	;  | chase mario and turn around if far enough away
	BPL +		;  | 
	TYA		;  | 
	EOR #$01	;  | 
	STA $157C,x	; /
+	LDA $14E0,x	; \
	XBA		;  | 
	LDA $E4,x	;  | 
	REP #$20	;  | 
	SEC		;  | 
	SBC $94		;  | branch if not close enough horizontally
	CLC		;  | 
	ADC #$0020	;  | 
	CMP #$0040	;  | 
	SEP #$20	;  | 
	BCS +		; /
	LDA $14D4,x	; \
	XBA		;  | 
	LDA $D8,x	;  | 
	REP #$20	;  | 
	SEC		;  | branch if not close enough vertically
	SBC $96		;  | 
	CMP #$0048	;  | 
	SEP #$20	;  | 
	BCS +		; /
	LDA #$D8	; \ set y speed
	STA $AA,x	; /
	LDA $B6,x	; \
	ASL		;  | cut some of sprite's velocity
	ROR $B6,x	; /
	LDA #$01	; \ sprite is in "lunging" state
	STA $1594,x	; /

	JSR WaterSplash	; display water splash
+	LDY $157C,x	; \
	LDA $1534,x	;  | 
	BEQ +		;  | 
	INY : INY	;  | 
+	LDA $B6,x	;  | 
	CLC		;  | accelerate sprite based on direction
	ADC .xvel_limit,y; | 
	BMI +		;  | 
	LDA $B6,x	;  | 
	CLC		;  | 
	ADC .xaccel,y	;  | 
	STA $B6,x	; /
+	RTS		; 
.jumpingup
	INC $1602,x	; lunging frame
	REP #$20	; \
	LDA $0E		;  | 
	SEC		;  | branch if sprite is above surface
	SBC $0C		;  | 
	SEP #$20	;  | 
	BCS +		; /
	LDA $AA,x	; \ branch if sprite is moving up
	BMI +		; /
	LDA #$18	; \ set y speed
	STA $AA,x	; /
	INC $1594,x	; sprite is in "resurfacing" state
	JSR WaterSplash	; display water splash
+	RTS
.resurface
	REP #$20	; \
	LDA $0E		;  | 
	SEC		;  | branch if sprite is below surface
	SBC $0C		;  | 
	SEP #$20	;  | 
	BCC +		; /
	LDA $AA,x	; \ branch if sprite is moving down
	BPL +		; /
	STZ $1594,x	; sprite is in "chasing" state
+	RTS		; 
.yaccel
	db $00,$02,$FE
.yvel_limit
	db $00,$30,$20
.xvel_limit
	db $30,$50,$20,$60
.xaccel	db $FC,$04,$FF,$01
.x_diff	db $10,$70

WaterSplash:
	LDA $1534,x	; \ branch if Big Bertha
	BEQ .bigbertha	; /
	LDA #$02	; \ set index to splash coordinate
	STA $00		; /
	BRA +		; 
.bigbertha
	STZ $00		; splash coordinate = 0
	JSR +		; 
	INC $00		; splash coordinate = 1
+	LDY #$0B	; \
-	LDA $17F0,y	;  | 
	BEQ +		;  | branch if free slot, loop otherwise
	DEY		;  | 
	BPL -		; /
	INY		; if no free slots, create one
+	LDA $D8,x	; \
	CLC		;  | 
	ADC #$F4	;  | 
	STA $17FC,y	;  | sprite y position = splash y position
	LDA $14D4,x	;  | 
	ADC #$FF	;  | 
	STA $1814,y	; /
	PHY		; \
	LDY $00		;  | 
	LDA $E4,x	;  | 
	CLC		;  | 
	ADC .xpos,y	;  | 
	PLY		;  | 
	STA $1808,y	;  | sprite x position + variable = splash x position
	PHY		;  | 
	LDY $00		;  | 
	LDA $14E0,x	;  | 
	ADC .xpos2,y	;  | 
	PLY		;  | 
	STA $18EA,y	; /
	LDA #$07	; \ sprite type
	STA $17F0,y	; /
	LDA #$00	; \ timer for splash
	STA $1850,y	; /
	RTS		; 
.xpos	db $FC,$04,$00
.xpos2	db $FF,$00,$00

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine

SubGfx:	JSR GET_DRAW_INFO       ; sets y = OAM offset

	STZ $04
	LDA $14C8,x
	CMP #$08
	BEQ +
	LDA #$05
	STA $04
+

	LDA $1534,x
	BEQ .bigbertha

	LDA $157C,x
	STA $02

	LDA $1570,x
	LSR : LSR : LSR
	AND #$01
	STA $03

	PHX
	LDA $00
	STA $0300,y
	LDA $01
	STA $0301,y
	LDX $03
	LDA .tilemap2,x
	STA $0302,y

	LDX $15E9
	LDA $15F6,x
	LDX $02
	BEQ +
	ORA #$40
+	LDX $04
	BEQ +
	ORA #$80
+	ORA $64
	STA $0303,y
	PLX

	LDY #$02
	LDA #$00
	JMP .finishoam

.bigbertha
	LDA $157C,x
	ASL : ASL
	ADC $157C,x
	STA $02

	PHY
	LDY $1602,x
	LDA $1570,x
	LSR : LSR : LSR
	AND #$01
	BNE +
	INY : INY
+	TYA
	PLY
	STA $03
	ASL : ASL
	ADC $03
	STA $03

	PHX
	LDX #$04
-	PHX
	TXA
	CLC
	ADC $02
	TAX
	LDA .xpos,x
	PLX
	CLC
	ADC $00
	STA $0300,y

	PHX
	TXA
	CLC
	ADC $04
	TAX
	LDA .ypos,x
	PLX
	CLC
	ADC $01
	STA $0301,y

	PHX
	TXA
	CLC
	ADC $03
	TAX
	LDA .tilemap,x
	PLX
	STA $0302,y

	PHX
	LDX $15E9
	LDA $15F6,x
	LDX $02
	BEQ +
	ORA #$40
+	LDX $04
	BEQ +
	ORA #$80
+	PLX
	ORA $64
	STA $0303,y

	PHY
	TYA
	LSR : LSR
	TAY
	LDA .tilesize,x
	STA $0460,y
	PLY

	INY : INY : INY : INY
	DEX
	BPL -
	PLX
		LDY #$FF
		LDA #$04
.finishoam	JSL $01B7B3
		RTS

.xpos	db $00,$08,$00,$10,$10
	db $08,$00,$08,$00,$00
.ypos	db $FA,$FA,$0A,$0A,$12
	db $0A,$0A,$FA,$02,$FA
.tilesize
	db $02,$02,$02,$00,$00
.tilemap
	db $E0,$E1,$E4,$E8,$F8
	db $E0,$E1,$E6,$E8,$F8
	db $E0,$E1,$E4,$E9,$F9
	db $E0,$E1,$E6,$E9,$F9

.tilemap2
	db $67,$69

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;       Y = index to sprite OAM ($300)
;       $00 = sprite x position relative to screen border
;       $01 = sprite y position relative to screen border  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1:             db $0C,$1C
SPR_T2:             db $01,$02

GET_DRAW_INFO:      STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X:        LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.w #$0040            ;  |
                    CMP.w #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP:     LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y:        DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID:            PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPR_T12:            db $40,$B0
SPR_T13:            db $01,$FF
SPR_T14:            db $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
		            db $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
SPR_T15:            db $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
					db $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SUB_OFF_SCREEN_X1:  LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2:  LDA #$04                ;  | 
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3:  LDA #$06                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4:  LDA #$08                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5:  LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6:  LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7:  LDA #$0E                ;  |
STORE_03:  STA $03					;  |            
					BRA START_SUB			;  |
SUB_OFF_SCREEN_X0:  STZ $03					; /

START_SUB:          JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ; | 
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; | 
                    CMP #$02                ; | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31:            LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE:       LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /    
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE:        STZ $14C8,x             ; erase sprite
RETURN_35:          RTS                     ; return

VERTICAL_LEVEL:     LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; | 
                    BCS RETURN_35           ; /
                    LDA $E4,x               ; \ 
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.w $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38:            LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

SUB_IS_OFF_SCREEN:  LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ; |  
                    RTS                     ; / return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_HORZ_POS + SUB_VERT_POS
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B817
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_HORZ_POS:	LDY #$00				;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
		LDA $94					;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
		SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
		SBC $E4,x				;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
		STA $0E					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
		LDA $95					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
		SBC $14E0,x				;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
		BPL +                   ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
		INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
+		RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642

SUB_VERT_POS:	LDY #$00				;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
		LDA $96					;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
		SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
		SBC $D8,x				;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
		STA $0F					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
		LDA $96					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
		SBC $14D4,x				;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
		BPL +                   ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
		INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
+		RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642